/**
 * Copyright Openmind http://www.openmindonline.it
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package it.openutils.mgnltasks;

import info.magnolia.cms.core.HierarchyManager;
import info.magnolia.importexport.BootstrapUtil;
import info.magnolia.module.InstallContext;
import info.magnolia.module.delta.BootstrapResourcesTask;
import info.magnolia.module.delta.TaskExecutionException;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import javax.jcr.ImportUUIDBehavior;
import javax.jcr.RepositoryException;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;


/**
 * A task to bootstrap a module. Can be used to fully re-bootstrap a custom module after an update (useful to reload
 * templates, dialogs, etc). This task will always ignore bootstrap files for the <code>website</code> repository.
 * <strong> Please note that during an update this task will always delete the following nodes inside the affected
 * module configuration: "dialogs", "templates", "paragraphs", "virtualURIMapping" </strong>
 * @author fgiust
 * @version $Id: ModuleConfigBootstrapTask.java 1516 2009-12-20 21:08:38Z fgiust $
 */
public class ModuleConfigBootstrapTask extends BootstrapResourcesTask
{

    /**
     * Logger.
     */
    private Logger log = LoggerFactory.getLogger(ModuleConfigBootstrapTask.class);

    private String modulename;

    private List<String> excludeRepositories = new ArrayList<String>();

    public ModuleConfigBootstrapTask(String modulename)
    {
        super("Bootstrap", "Bootstraps module configuration for " + modulename + " (will not overwrite website!).");
        this.modulename = modulename;
        excludeRepositories.add("website");
    }

    public ModuleConfigBootstrapTask(String modulename, List<String> excludeRepositories)
    {
        super("Bootstrap", "Bootstraps module configuration for " + modulename + " (will not overwrite website!).");
        this.modulename = modulename;
        this.excludeRepositories.addAll(excludeRepositories);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    protected boolean acceptResource(InstallContext ctx, String name)
    {

        boolean accept = name.startsWith("/mgnl-bootstrap/" + modulename + "/") && name.endsWith(".xml");

        for (String repository : excludeRepositories)
        {
            accept = accept && !name.startsWith("/mgnl-bootstrap/" + modulename + "/" + repository);
        }

        if (accept)
        {
            log.debug("Importing file {}", name);
        }
        return accept;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void execute(final InstallContext installContext) throws TaskExecutionException
    {
        try
        {
            final String[] resourcesToBootstrap = getResourcesToBootstrap(installContext);

            long millis = System.currentTimeMillis();
            log.info("{} bootstrap starting...", modulename);

            deleteNode(installContext, "/modules/" + modulename + "/dialogs");
            deleteNode(installContext, "/modules/" + modulename + "/templates");
            deleteNode(installContext, "/modules/" + modulename + "/paragraphs");
            deleteNode(installContext, "/modules/" + modulename + "/virtualURIMapping");

            BootstrapUtil.bootstrap(resourcesToBootstrap, ImportUUIDBehavior.IMPORT_UUID_COLLISION_REMOVE_EXISTING);
            log.info("{} bootstrap done in {} seconds", modulename, (System.currentTimeMillis() - millis) / 1000);
        }
        catch (IOException e)
        {
            throw new TaskExecutionException("Could not bootstrap: " + e.getMessage());
        }
        catch (RepositoryException e)
        {
            throw new TaskExecutionException("Could not bootstrap: " + e.getMessage());
        }
    }

    /**
     * Deletes a node
     * @param installContext install context
     * @param nodePath node path
     * @throws RepositoryException for any exception wile operating on the repository
     */
    protected void deleteNode(InstallContext installContext, String nodePath) throws RepositoryException
    {

        HierarchyManager hm = installContext.getHierarchyManager("config");

        if (hm.isExist(nodePath))
        {
            log.warn("Deleting node {}", nodePath);
            hm.delete(nodePath);
        }
    }
}